﻿using System;
using System.IO;
using JetBrains.Annotations;
using Microsoft.Build.Framework;
using Microsoft.Build.Utilities;
using Sda.DictionaryItemEnumGenerator;

namespace Sda.Build.Tasks
{
    /// <summary>
    /// Kódtételek generálása.
    /// </summary>
    /// <remarks>
    /// Oracle kliens binárisok legyenek a <c>PATH</c>-ban!
    /// </remarks>
    public sealed class DictionaryItem : Task
    {
        /// <summary>
        /// Kréta adatbázis script - szótár elemeket felvevő script
        /// </summary>
        [CanBeNull, Required]
        public string ScriptOutputDir { get; set; }

        /// <summary>
        /// Kódtétel osztályokat listázó fájl.
        /// </summary>
        [CanBeNull, Required]
        public string InputFile { get; set; }

        /// <summary>
        /// C# alkalmazás-kiszolgáló kimeneti könyvtár.
        /// </summary>
        [CanBeNull, Required]
        public string CSharpOutputDir { get; set; }

        /// <summary>
        /// A Solutation alatt a GeneratedDictionaryItems.XML fájl.
        /// </summary>
        [CanBeNull, Required]
        public string InputDictionaryXMLFile { get; set; }

        public override bool Execute()
        {
            if (string.IsNullOrEmpty(ScriptOutputDir))
            {
                return false;
            }
            if (string.IsNullOrEmpty(InputFile)
                ||
                !File.Exists(InputFile))
            {
                return false;
            }
            if (string.IsNullOrEmpty(InputDictionaryXMLFile)
    ||
    !File.Exists(InputDictionaryXMLFile))
            {
                return false;
            }
            Generator.Generate(
                InputFile,
                InputDictionaryXMLFile,
                ScriptOutputDir,
                Path.Combine(CSharpOutputDir ?? ".", "GeneratedEnums.cs"));
            return true;
        }
    }
}
