﻿using System.Diagnostics;
using System.Linq;
using JetBrains.Annotations;
using Microsoft.Build.Framework;
using Microsoft.Build.Utilities;
using Sda.Modeling.Runtime;

namespace Sda.Build.Tasks
{
    /// <summary>
    /// <c>IMF</c> model generálás <c>XMI</c> alapján.
    /// </summary>
    public sealed class GenerateImf : Task
    {
        /// <summary>
        /// A bemeneti <c>XMI</c> állományok nevei, első legyen a gyökér projekt.
        /// </summary>
        [CanBeNull, Required]
        public ITaskItem[] Inputs { get; set; }

        /// <summary>
        /// A kimeneti <c>IMF</c> állomány neve.
        /// </summary>
        [CanBeNull, Required]
        public string Output { get; set; }

        public override bool Execute()
        {
            if (string.IsNullOrEmpty(Output))
            {
                return false;
            }
            if (Inputs == null)
            {
                return false;
            }
            Generator generator = new Generator();
            generator.LogMessageHandler += (sender, args) =>
            {
                switch (args.Type)
                {
                    case TraceEventType.Error:
                        this.LogError("GenerateImf: {0}", args.Message);
                        break;
                    case TraceEventType.Warning:
                        this.LogWarning("GenerateImf: {0}", args.Message);
                        break;
                    default:
                        this.LogInfo("GenerateImf: {0}", args.Message);
                        break;
                }
            };
            generator.Run(Output, Inputs.Select(i => i.ItemSpec).ToArray());
            return true;
        }
    }
}
