﻿using System;
using System.Diagnostics.CodeAnalysis;
using System.Text;
using JetBrains.Annotations;
using Microsoft.Build.Framework;
using Microsoft.Build.Utilities;

namespace Sda.Build.Tasks
{
    /// <summary>
    /// <c>DOS</c> környezeti változó deklarálás.
    /// </summary>
    [SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Env")]
    public sealed class SetEnvVar : Task
    {
        /// <summary>
        /// A környezeti változó neve.
        /// </summary>
        [CanBeNull, Required]
        public string Variable { get; set; }

        /// <summary>
        /// A korábbi értékhez karakteres hozzáfűzés történjen, pl. <c>%PATH%</c> esetén hasznos.
        /// </summary>
        public bool Append { get; set; }

        /// <summary>
        /// Környezeti változó értéke.
        /// </summary>
        [CanBeNull, Required]
        public string Value { get; set; }

        public override bool Execute()
        {
            if (Variable == null)
            {
                return false;
            }
            if (Append)
            {
                StringBuilder sb = new StringBuilder(Environment.GetEnvironmentVariable(Variable) ?? "");
                sb.Append(Value);
                Environment.SetEnvironmentVariable(Variable, sb.ToString());
            }
            else
            {
                Environment.SetEnvironmentVariable(Variable, Value);
            }
            return true;
        }
    }
}
