﻿using System;
using System.IO;
using System.Linq;
using JetBrains.Annotations;
using Microsoft.Build.Framework;
using Microsoft.Build.Utilities;

namespace Sda.Build.Tasks
{
    public sealed class UnixLineEnding : Task
    {
        /// <summary>
        /// Kréta adatbázis script - szótár elemeket felvevő script
        /// </summary>
        [CanBeNull, Required]
        public string DataAccessorsPath { get; set; }

        /// <summary>
        /// Kódtétel osztályokat listázó fájl.
        /// </summary>
        [CanBeNull, Required]
        public string EntitiesPath { get; set; }

        /// <summary>
        /// C# alkalmazás-kiszolgáló kimeneti könyvtár.
        /// </summary>
        [CanBeNull, Required]
        public string InterfacesPath { get; set; }

        public override bool Execute()
        {
            if (string.IsNullOrEmpty(DataAccessorsPath) || string.IsNullOrEmpty(EntitiesPath) || string.IsNullOrEmpty(InterfacesPath))
            {
                return false;
            }
            if (!Directory.Exists(DataAccessorsPath) || !Directory.Exists(EntitiesPath) || !Directory.Exists(InterfacesPath))
            {
                return false;
            }
            var files = Directory.GetFiles(DataAccessorsPath, "*.cs").ToList();
            files.AddRange(Directory.GetFiles(EntitiesPath, "*.cs"));
            files.AddRange(Directory.GetFiles(InterfacesPath, "*.cs"));
            foreach (var file in files)
            {
                var content = File.ReadAllBytes(file);
                var newContent = Array.FindAll(content, c => c != 0x0d);
                File.WriteAllBytes(file, newContent);
            }
            return true;
        }
    }
}
