﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using JetBrains.Annotations;

namespace Sda.DictionaryItemEnumGenerator
{
    public class DictionaryItemGroup
    {
        public DictionaryItemGroup()
        {
            Items = new List<DictionaryItem>();
        }
        public int Id { get; set; }

        public string TableName { get; set; }

        public string Code { get; set; }

        //public int ResourceId { get; set; }

        public bool CustomItemAllowed { get; set; }

        public bool ColorEditable { get; set; }

        public string Name { get; set; }
        public string Name1 { get; set; }
        public string Name2 { get; set; }
        public string Name3 { get; set; }
        public string Name4 { get; set; }

        public List<DictionaryItem> Items { get; set; }
    }

    public class DictionaryItem
    {
        public int Id { get; set; }
        public string EnumName { get; set; }
        public List<DictionaryItemValue> DictionaryItemValues { get; set; } = new List<DictionaryItemValue>();
    }

    public class DictionaryItemValue
    {
        public string EnvironmentName { get; set; }
        //public int StringResourcesId { get; set; }

        /// <summary>
        /// Magyar
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// Angol
        /// </summary>
        public string Name1 { get; set; }
        public string Name2 { get; set; }
        public string Name3 { get; set; }
        public string Name4 { get; set; }

        public bool Protected { get; set; }
        public bool Visible { get; set; }

        public int? Order { get; set; }
        public string Code { get; set; }
        public int? Value { get; set; }
        public string Color { get; set; }
    }

    public class DictionaryItemGroupUtils
    {
        public List<DictionaryItemGroup> itemGroups { get; set; }

        #region Load
        /// <summary>
        /// A TXT és XML fájl adatok összefésülése.
        /// </summary>
        /// <param name="idc"></param>
        public void Load(InputDataCollection idc, List<DictionaryItemGroup> xmlData)
        {
            itemGroups = new List<DictionaryItemGroup>();

            for (int index = 0; index < idc.Count; index++)
            {
                var umlDictionaryItem = idc[index];

                // megkeresem hozzá az XML adatokat
                var xmlDataItem = xmlData.Where(d => d.Code == umlDictionaryItem.DictionaryItemGroupName).SingleOrDefault();
                if (xmlDataItem != null)
                {
                    xmlDataItem.TableName = umlDictionaryItem.TableName;
                    itemGroups.Add(xmlDataItem);
                }
                //else
                //    throw new ArgumentNullException(string.Format("{0} "));
            }
        }

        #endregion

        #region Save

        public void SaveCSharp([NotNull] string outputFile)
        {
            if (string.IsNullOrEmpty(outputFile))
                throw new ArgumentNullException(nameof(outputFile));
            if (itemGroups == null || itemGroups.Count == 0)
                throw new ArgumentNullException(nameof(itemGroups));
            string adatszotarTipusEnumOutputFile = outputFile.Replace("GeneratedEnums.cs", @"GeneratedAdatszotarTipusEnum.cs");
            var file = new StringBuilder().AppendFormat(
@"//------------------------------------------------------------------------------
// <auto-generated>
//     This code was generated by a tool.
//
//     Changes to this file may cause incorrect behavior and will be lost if
//     the code is regenerated.
// </auto-generated>
//------------------------------------------------------------------------------
{0}",
                GenerateEnums());
            using (StreamWriter writer = new StreamWriter(Path.Combine(Path.GetDirectoryName(outputFile), "GeneratedEnums.cs"), false, Encoding.UTF8))
            {
                writer.Write(Regex.Replace(file.ToString(), "((?<!\r)\n|\r(?!\n))", "\r\n"));
            }
            file = new StringBuilder().AppendFormat(
@"//------------------------------------------------------------------------------
// <auto-generated>
//     This code was generated by a tool.
//
//     Changes to this file may cause incorrect behavior and will be lost if
//     the code is regenerated.
// </auto-generated>
//------------------------------------------------------------------------------
{0}",
                GenerateAdatszotarEnum());
            using (StreamWriter writer = new StreamWriter(Path.Combine(Path.GetDirectoryName(adatszotarTipusEnumOutputFile), "GeneratedAdatszotarTipusEnum.cs"), false, Encoding.UTF8))
            {
                writer.Write(Regex.Replace(file.ToString(), "((?<!\r)\n|\r(?!\n))", "\r\n"));
            }
        }

        #endregion

        #region Generate

        [NotNull]
        string GenerateEnums()
        {
            StringBuilder builder = new StringBuilder();
            string nsp = "";
            foreach (DictionaryItemGroup group in this.itemGroups)
            {
                builder.Append(GenerateEnumWithNameSpace(group, ref nsp));
            }
            if (builder.Length > 0)
            {
                builder.Append(@"
}
");
            }
            return builder.ToString();
        }

        [NotNull]
        string GenerateAdatszotarEnum()
        {
            string nsp = "Kreta.Enums";
            string header = "public enum GeneratedAdatszotarTipusEnum";

            var body = new StringBuilder();
            int valueIndex = 1;
            foreach (DictionaryItemGroup group in this.itemGroups.OrderBy(x => x.Id))
            {
                var lastLine = valueIndex == this.itemGroups.Count;
                body.AppendFormat(
                    "{0}\r\n", GenerateAdatszotarEnumLine(group, valueIndex, lastLine));

                valueIndex++;
            }
            return string.Format(@"
namespace {0}
{{
	{1}
	{{{2}
	}}
}}
",
                    nsp,
                    header,
                    body);
        }


        [NotNull]
        string GenerateEnumWithNameSpace([NotNull] DictionaryItemGroup group, [CanBeNull] ref string nsp)
        {
            string str = GenerateEnum(group);
            string ns = "Kreta.Enums";
            if (string.IsNullOrEmpty(nsp))
            {
                nsp = ns;
                return string.Format(@"
namespace {0}
{{{1}",
                    ns,
                    str);
            }
            if (ns == nsp)
            {
                return string.Format(@"
{0}",
                    str);
            }
            nsp = ns;
            return string.Format(@"
}}

namespace {0}
{{{1}",
                ns,
                str);
        }

        [NotNull]
        string GenerateEnum([NotNull] DictionaryItemGroup group)
        {
            string comment = string.Format(@"
	/// <summary>
	/// <entity>{0}</entity>
	/// <description>{1}</description>
	/// <tablename>{2}</tablename>
    /// <protected>{3}</protected>
	/// </summary>",
                (group.Code ?? "").Trim(),
                string.IsNullOrWhiteSpace(group.Name) ? "Nincs leírás." : (group.Name ?? "").Trim(),
                (group.TableName ?? "").Trim(), !group.CustomItemAllowed ? "Igen" : "Nem");

            string header = string.Format(
                "\tpublic enum {0}Enum",
                group.Code.ToBase26String());

            var body = new StringBuilder();
            int valueIndex = 1;
            foreach (var itemValue in group.Items)
            {
                var lastLine = valueIndex == group.Items.Count;
                body.AppendFormat(
                    "{0}\r\n", GenerateEnumLine(itemValue, valueIndex, lastLine));

                valueIndex++;
            }
            return string.Format(
                "{0}\r\n\t{1}\r\n\t{{{2}\t}}",
                comment,
                header,
                body);
        }

        #endregion

        #region Generate line

        [NotNull]
        string GenerateEnumLine(DictionaryItem item, int valueIndex, bool LastItem)
        {
            string comment = string.Format(
                "Name: {0}",
                item.DictionaryItemValues.First(x => x.EnvironmentName == "Default").Name);
            string comma = "";
            if (!LastItem)
            {
                comma = ",";
            }


            return string.Format(@"
		/// <summary>
		/// {0}
		/// </summary>
        {1} = {2}{3}",
    comment,
    item.EnumName.ToBase26String(),
    item.Id,
    comma);

        }

        [NotNull]
        string GenerateAdatszotarEnumLine(DictionaryItemGroup group, int valueIndex, bool LastItem)
        {
            string comma = "";
            if (!LastItem)
            {
                comma = ",";
            }


            return string.Format(@"
		{0} = {1}{2}",
    group.Code.ToBase26String(),
    group.Id,
    comma);

        }

        #endregion

        #region Generate Scripts

        public void GenerateDBScripts([NotNull] string outputScriptPath)
        {
            if (string.IsNullOrEmpty(outputScriptPath))
                throw new ArgumentNullException(nameof(outputScriptPath));

            if (itemGroups == null || itemGroups.Count == 0)
                throw new ArgumentNullException(nameof(itemGroups));

            var file = GetDBScripts();
            using (StreamWriter writer = new StreamWriter(Path.Combine(Path.GetDirectoryName(outputScriptPath), "uspCreateDictionaryItems.sql"), false, Encoding.UTF8))
            {
                writer.Write(Regex.Replace(file.ToString(), "((?<!\r)\n|\r(?!\n))", "\r\n"));
            }

        }

        private string GetDBScripts()
        {
            string KK_EnvironmentName = "KK";
            string NSZFH_EnvironmentName = "NSZFH";
            string HOI_EnvironmentName = "HOI";
            string AZURE_EnvironmentName = "AZURE";
            string Default_EnvironmentName = "Default";
            StringBuilder script = new StringBuilder(@"
-- =============================================
-- Description:	Szótár elemeket létrehozó script 
-- =============================================

DROP PROCEDURE IF EXISTS [dev].uspCreateDictionaryItems
GO

CREATE PROCEDURE [dev].uspCreateDictionaryItems
    @IntezmenyId int = NULL,
    @pTanevNev nvarchar(10) = NULL
AS
BEGIN
  SET NOCOUNT ON;
  DECLARE @EnvironmentName nvarchar(1000), @LastTanevSorszam int
  SET @EnvironmentName = (SELECT [Value] FROM dev.EnvironmentInfo WHERE [Key] = 'Dictionary')

  IF (@EnvironmentName IS NULL OR @EnvironmentName NOT IN ('KK','NSZFH','HOI','AZURE'))
  BEGIN  INSERT INTO T_DICTIONARYITEMBASE (C_VALUE) VALUES ('Nincs Dictionary ertek a dev.EnvironmentInfo tablaban') END -- elszall a migracio ha nincs
  
  SET @LastTanevSorszam  = (SELECT MAX(C_SORSZAM) FROM T_TANEV WHERE TOROLT = 'F' AND (C_KOVETKEZO = 'T' OR C_AKTIV = 'T') AND @IntezmenyId IS NULL OR C_INTEZMENYID = @IntezmenyId)

  DROP TABLE IF EXISTS #Tanevek
  CREATE TABLE #Tanevek (IntezmenyId int, TanevId int)
  IF @IntezmenyId IS NULL BEGIN
      INSERT INTO #Tanevek (IntezmenyId, TanevId)
      SELECT
         i.ID
        ,t.ID
      FROM T_INTEZMENY i
        INNER JOIN T_TANEV t ON t.C_INTEZMENYID = i.ID
          AND C_SORSZAM <= @LastTanevSorszam
          AND (@pTanevNev IS NULL OR t.C_NEV = @pTanevNev)
      WHERE i.C_ISARCHIV = 'F'
    END
    ELSE BEGIN
      INSERT INTO #Tanevek (IntezmenyId, TanevId)
      SELECT
         i.ID
        ,t.ID
      FROM T_INTEZMENY i
        INNER JOIN T_TANEV t ON t.C_INTEZMENYID = i.ID
          AND t.C_INTEZMENYID = i.ID
          AND C_SORSZAM <= @LastTanevSorszam
          AND (@pTanevNev IS NULL OR t.C_NEV = @pTanevNev)
      WHERE i.ID = @IntezmenyId
        AND i.C_ISARCHIV = 'F'
    END
    
  DROP TABLE IF EXISTS #DICTIONARYTYPE
  CREATE TABLE #DICTIONARYTYPE(
     ID INT PRIMARY KEY
    ,C_CODENAME NVARCHAR(255) INDEX IX_CODENAME NONCLUSTERED
    ,C_NAME NVARCHAR(255)
    ,C_ISCUSTOMITEMALLOWED NVARCHAR(1)
    ,C_ISCOLOREDITABLE NVARCHAR(1)
    ,C_DESCRIPTION NVARCHAR(255)
  )
  DROP TABLE IF EXISTS #DICTIONARYITEMBASE
  CREATE TABLE #DICTIONARYITEMBASE (
     ID INT not null PRIMARY KEY
    ,C_NAME NVARCHAR(250) not null
    ,C_NAME_1 NVARCHAR(250) null
    ,C_NAME_2 NVARCHAR(250) null
    ,C_NAME_3 NVARCHAR(250) null
    ,C_NAME_4 NVARCHAR(250) null
    ,C_TYPE NVARCHAR(32) not null INDEX IX_TYPE NONCLUSTERED
    ,C_DICTIONARYTYPEID INT not null
    ,C_PROTECTED CHAR(1) null
    ,C_VISIBLE CHAR(1) not null
    ,C_ORDER INT null
    ,C_CODE NVARCHAR(20) null
    ,C_VALUE INT null
    ,C_COLOR NVARCHAR(6) null
  )

    -- DictionaryType elemek 

    INSERT INTO #DICTIONARYTYPE (ID, C_CODENAME, C_NAME, C_ISCUSTOMITEMALLOWED, C_ISCOLOREDITABLE, C_DESCRIPTION) 
        VALUES");

            foreach (var itemGroup in itemGroups)
            {
                script.AppendFormat(@"
            ({0}, N'{1}', N'{2}', '{3}', '{4}', N'{5}'),",
                    itemGroup.Id,
                    itemGroup.Code,
                    itemGroup.Name,
                    itemGroup.CustomItemAllowed ? "T" : "F",
                    itemGroup.ColorEditable ? "T" : "F",
                    "NULL");
            }

            script.Remove(script.Length - 1, 1);

            script.Append(@"
    SET IDENTITY_INSERT T_DICTIONARYTYPE ON

    INSERT INTO T_DICTIONARYTYPE (ID, C_CODENAME, C_NAME, C_ISCUSTOMITEMALLOWED, C_ISCOLOREDITABLE, C_DESCRIPTION, C_INTEZMENYID, C_TANEVID)
      SELECT
         ID
        ,C_CODENAME
        ,C_NAME
        ,C_ISCUSTOMITEMALLOWED
        ,C_ISCOLOREDITABLE
        ,C_DESCRIPTION
        ,tn.IntezmenyId AS C_INTEZMENYID
        ,tn.TanevId AS C_TANEVID
      FROM #DICTIONARYTYPE tdt
        CROSS JOIN #Tanevek tn
      WHERE NOT EXISTS (SELECT 1 FROM dbo.T_DICTIONARYTYPE dt WHERE dt.ID = tdt.ID AND dt.C_INTEZMENYID = tn.IntezmenyId AND dt.C_TANEVID = tn.TanevId) 

    SET IDENTITY_INSERT T_DICTIONARYTYPE OFF

    -- DictionaryItemBase elemek 
    ");

            if (itemGroups.Exists(x => x.Items.Exists(y => y.DictionaryItemValues.Exists(z => z.EnvironmentName == KK_EnvironmentName))))
            {
                script.Append(@"
    -- A '" + KK_EnvironmentName + @"' sajatos elemek kezdete
    IF (@EnvironmentName IS NOT NULL AND @EnvironmentName = '" + KK_EnvironmentName + @"')
    BEGIN 
    ");

                int index = 0;
                foreach (var itemGroup in itemGroups)
                {
                    var order = 1;
                    foreach (var item in itemGroup.Items)
                    {
                        foreach (var value in item.DictionaryItemValues.Where(x => x.EnvironmentName == KK_EnvironmentName))
                        {

                            if ((index % 900) == 0)
                            {
                                script.Remove(script.Length - 1, 1);
                                script.Append(@"
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        VALUES");

                            }

                            script.AppendFormat(@"
            ({0}, N'{1}', {2}, {3}, {4}, {5}, N'{6}', {7}, N'{8}', N'{9}', {10}, {11}, {12}, {13}),",
                                item.Id,
                                value.Name,
                                string.IsNullOrEmpty(value.Name1) == false ? "N'" + value.Name1 + "'" : "NULL",
                                string.IsNullOrEmpty(value.Name2) == false ? "N'" + value.Name2 + "'" : "NULL",
                                string.IsNullOrEmpty(value.Name3) == false ? "N'" + value.Name3 + "'" : "NULL",
                                string.IsNullOrEmpty(value.Name4) == false ? "N'" + value.Name4 + "'" : "NULL",
                                itemGroup.Code,
                                itemGroup.Id,
                                value.Protected ? "T" : "F",
                                value.Visible ? "T" : "F",
                                value.Order ?? order,
                                string.IsNullOrEmpty(value.Code) == false ? "N'" + value.Code + "'" : "NULL",
                                value.Value ?? order,
                                string.IsNullOrEmpty(value.Color) == false ? "N'" + value.Color + "'" : "NULL");

                            order++;
                            index++;
                        }
                    }
                }

                script.Remove(script.Length - 1, 1);
                script.AppendLine(@"
    END
    -- A '" + KK_EnvironmentName + @"' sajatos elemek vege");
            }
            if (itemGroups.Exists(x => x.Items.Exists(y => y.DictionaryItemValues.Exists(z => z.EnvironmentName == NSZFH_EnvironmentName))))
            {
                script.Append(@"
    -- A '" + NSZFH_EnvironmentName + @"' sajatos elemek kezdete
    IF (@EnvironmentName IS NOT NULL AND @EnvironmentName = '" + NSZFH_EnvironmentName + @"')
    BEGIN 
    ");

                int index = 0;
                foreach (var itemGroup in itemGroups)
                {
                    var order = 1;
                    foreach (var item in itemGroup.Items)
                    {
                        foreach (var value in item.DictionaryItemValues.Where(x => x.EnvironmentName == NSZFH_EnvironmentName))
                        {

                            if ((index % 900) == 0)
                            {
                                script.Remove(script.Length - 1, 1);
                                script.Append(@"
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        VALUES");

                            }

                            script.AppendFormat(@"
            ({0}, N'{1}', {2}, {3}, {4}, {5}, N'{6}', {7}, N'{8}', N'{9}', {10}, {11}, {12}, {13}),",
                                item.Id,
                                value.Name,
                                string.IsNullOrEmpty(value.Name1) == false ? "N'" + value.Name1 + "'" : "NULL",
                                string.IsNullOrEmpty(value.Name2) == false ? "N'" + value.Name2 + "'" : "NULL",
                                string.IsNullOrEmpty(value.Name3) == false ? "N'" + value.Name3 + "'" : "NULL",
                                string.IsNullOrEmpty(value.Name4) == false ? "N'" + value.Name4 + "'" : "NULL",
                                itemGroup.Code,
                                itemGroup.Id,
                                value.Protected ? "T" : "F",
                                value.Visible ? "T" : "F",
                                value.Order ?? order,
                                string.IsNullOrEmpty(value.Code) == false ? "N'" + value.Code + "'" : "NULL",
                                value.Value ?? order,
                                string.IsNullOrEmpty(value.Color) == false ? "N'" + value.Color + "'" : "NULL");

                            order++;
                            index++;
                        }
                    }
                }

                script.Remove(script.Length - 1, 1);
                script.AppendLine(@"
    END
    -- A '" + NSZFH_EnvironmentName + @"' sajatos elemek vege");
            }
            if (itemGroups.Exists(x => x.Items.Exists(y => y.DictionaryItemValues.Exists(z => z.EnvironmentName == HOI_EnvironmentName))))
            {
                script.Append(@"    
    -- A '" + HOI_EnvironmentName + @"' sajatos elemek kezdete
    IF (@EnvironmentName IS NOT NULL AND @EnvironmentName = '" + HOI_EnvironmentName + @"')
    BEGIN 
    ");
                int index = 0;
                foreach (var itemGroup in itemGroups)
                {
                    var order = 1;
                    foreach (var item in itemGroup.Items)
                    {
                        foreach (var value in item.DictionaryItemValues.Where(x => x.EnvironmentName == HOI_EnvironmentName))
                        {

                            if ((index % 900) == 0)
                            {
                                script.Remove(script.Length - 1, 1);
                                script.Append(@"
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        VALUES");

                            }

                            script.AppendFormat(@"
            ({0}, N'{1}', {2}, {3}, {4}, {5}, N'{6}', {7}, N'{8}', N'{9}', {10}, {11}, {12}, {13}),",
                                item.Id,
                                value.Name,
                                string.IsNullOrEmpty(value.Name1) == false ? "N'" + value.Name1 + "'" : "NULL",
                                string.IsNullOrEmpty(value.Name2) == false ? "N'" + value.Name2 + "'" : "NULL",
                                string.IsNullOrEmpty(value.Name3) == false ? "N'" + value.Name3 + "'" : "NULL",
                                string.IsNullOrEmpty(value.Name4) == false ? "N'" + value.Name4 + "'" : "NULL",
                                itemGroup.Code,
                                itemGroup.Id,
                                value.Protected ? "T" : "F",
                                value.Visible ? "T" : "F",
                                value.Order ?? order,
                                string.IsNullOrEmpty(value.Code) == false ? "N'" + value.Code + "'" : "NULL",
                                value.Value ?? order,
                                string.IsNullOrEmpty(value.Color) == false ? "N'" + value.Color + "'" : "NULL");

                            order++;
                            index++;
                        }
                    }
                }

                script.Remove(script.Length - 1, 1);
                script.AppendLine(@"
    END
    -- A '" + HOI_EnvironmentName + @"' sajatos elemek vege");
            }
            if (itemGroups.Exists(x => x.Items.Exists(y => y.DictionaryItemValues.Exists(z => z.EnvironmentName == AZURE_EnvironmentName))))
            {
                script.Append(@"        
    -- A '" + AZURE_EnvironmentName + @"' sajatos elemek kezdete
    IF (@EnvironmentName IS NOT NULL AND @EnvironmentName = 'AZURE')
    BEGIN 
    ");
                int index = 0;
                foreach (var itemGroup in itemGroups)
                {
                    var order = 1;
                    foreach (var item in itemGroup.Items)
                    {
                        foreach (var value in item.DictionaryItemValues.Where(x => x.EnvironmentName == AZURE_EnvironmentName))
                        {

                            if ((index % 900) == 0)
                            {
                                script.Remove(script.Length - 1, 1);
                                script.Append(@"
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        VALUES");

                            }

                            script.AppendFormat(@"
            ({0}, N'{1}', {2}, {3}, {4}, {5}, N'{6}', {7}, N'{8}', N'{9}', {10}, {11}, {12}, {13}),",
                                item.Id,
                                value.Name,
                                string.IsNullOrEmpty(value.Name1) == false ? "N'" + value.Name1 + "'" : "NULL",
                                string.IsNullOrEmpty(value.Name2) == false ? "N'" + value.Name2 + "'" : "NULL",
                                string.IsNullOrEmpty(value.Name3) == false ? "N'" + value.Name3 + "'" : "NULL",
                                string.IsNullOrEmpty(value.Name4) == false ? "N'" + value.Name4 + "'" : "NULL",
                                itemGroup.Code,
                                itemGroup.Id,
                                value.Protected ? "T" : "F",
                                value.Visible ? "T" : "F",
                                value.Order ?? order,
                                string.IsNullOrEmpty(value.Code) == false ? "N'" + value.Code + "'" : "NULL",
                                value.Value ?? order,
                                string.IsNullOrEmpty(value.Color) == false ? "N'" + value.Color + "'" : "NULL");

                            order++;
                            index++;
                        }
                    }
                }

                script.Remove(script.Length - 1, 1);
                script.AppendLine(@"
    END
    -- A '" + AZURE_EnvironmentName + @"' sajatos elemek vege");
            }
            script.Append(@"    
    -- A '" + Default_EnvironmentName + @"' elemek kezdete ");
            int indexdefault = 0;
            foreach (var itemGroup in itemGroups)
            {
                var order = 1;
                foreach (var item in itemGroup.Items)
                {
                    foreach (var value in item.DictionaryItemValues.Where(x => x.EnvironmentName == Default_EnvironmentName))
                    {
                        if ((indexdefault % 900) == 0)
                        {
                            script.Remove(script.Length - 1, 1);
                            script.Append(@"
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        SELECT * FROM ( VALUES");

                        }

                        script.AppendFormat(@"
            ({0}, N'{1}', {2}, {3}, {4}, {5}, N'{6}', {7}, N'{8}', N'{9}', {10}, {11}, {12}, {13}),",
                            item.Id,
                            value.Name,
                            string.IsNullOrEmpty(value.Name1) == false ? "N'" + value.Name1 + "'" : "NULL",
                            string.IsNullOrEmpty(value.Name2) == false ? "N'" + value.Name2 + "'" : "NULL",
                            string.IsNullOrEmpty(value.Name3) == false ? "N'" + value.Name3 + "'" : "NULL",
                            string.IsNullOrEmpty(value.Name4) == false ? "N'" + value.Name4 + "'" : "NULL",
                            itemGroup.Code,
                            itemGroup.Id,
                            value.Protected ? "T" : "F",
                            value.Visible ? "T" : "F",
                            value.Order ?? order,
                            string.IsNullOrEmpty(value.Code) == false ? "N'" + value.Code + "'" : "NULL",
                            value.Value ?? order,
                            string.IsNullOrEmpty(value.Color) == false ? "N'" + value.Color + "'" : "NULL");

                        if ((((indexdefault + 1) % 900) == 0) || (indexdefault + 1) == itemGroups.SelectMany(x => x.Items).SelectMany(x => x.DictionaryItemValues).Count(z => z.EnvironmentName == Default_EnvironmentName))
                        {
                            script.Remove(script.Length - 1, 1);
                            script.AppendLine(@"
                ) x (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
                WHERE NOT EXISTS (SELECT 1 FROM #DICTIONARYITEMBASE d WHERE x.ID = d.ID)
                ");
                        }

                        order++;
                        indexdefault++;
                    }
                }
            }

            script.AppendLine(@"
    -- A '" + Default_EnvironmentName + @"' elemek vege
    ");



            script.AppendLine(@"
    SET IDENTITY_INSERT [dbo].[T_DICTIONARYITEMBASE] ON
	INSERT INTO dbo.T_DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR, C_INTEZMENYID, C_TANEVID, TOROLT, SERIAL, LASTCHANGED, CREATED, MODIFIER, CREATOR, ELOZOTANEVIREKORDID)
		SELECT tdib.ID, tdib.C_NAME, tdib.C_NAME_1, tdib.C_NAME_2, tdib.C_NAME_3, tdib.C_NAME_4, tdib.C_TYPE, tdib.C_DICTIONARYTYPEID, tdib.C_PROTECTED, tdib.C_VISIBLE, tdib.C_ORDER, tdib.C_CODE, tdib.C_VALUE, tdib.C_COLOR, tn.IntezmenyId, tn.TanevId, 'F', 0, GETDATE(), GETDATE(), NULL, NULL,tdib.ID
		FROM #DICTIONARYITEMBASE tdib
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.T_DICTIONARYITEMBASE dib WHERE dib.ID = tdib.ID AND dib.C_INTEZMENYID = tn.IntezmenyId AND dib.C_TANEVID = tn.TanevId)");

            script.AppendLine(@"
    SET IDENTITY_INSERT [dbo].[T_DICTIONARYITEMBASE] OFF");

            script.Append(@"
    -- DictionaryItem elemek ");

            foreach (var itemGroup in itemGroups)
            {
                if (itemGroup.Items.Any())
                {

                    script.AppendFormat(@"

    INSERT INTO [dbo].[{0}] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[{0}] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = {1} ", itemGroup.TableName, itemGroup.Id);
                }
            }

            script.Append(@" DROP TABLE IF EXISTS #DICTIONARYITEMBASE

END");

            return script.ToString();
        }

        #endregion

    }
}



