﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Xml.Linq;
using JetBrains.Annotations;

namespace Sda.DictionaryItemEnumGenerator
{
    public class GeneratedDictionaryInputDataCollection
    {
        public GeneratedDictionaryInputDataCollection()
        {
            DictionaryItemGroups = new List<DictionaryItemGroup>();
        }
        public List<DictionaryItemGroup> DictionaryItemGroups { get; set; }

        /// <summary>
        /// DictionaryItemek beolvasasa
        /// </summary>
        /// <param name="fileName"></param>
        public void Load([NotNull] string fileName)
        {
            if (fileName == null)
                throw new ArgumentNullException(nameof(fileName));

            if (File.Exists(fileName))
            {
                // XML fájl beolvasása
                XElement root = XElement.Load(fileName);

                // dict.itemek feltöltése
                foreach (XElement dictionaryType in root.Elements())
                {
                    var dictionaryGroup = new DictionaryItemGroup
                    {
                        Id = Convert.ToInt32(dictionaryType.Attribute("Id").Value),
                        Code = dictionaryType.Attribute("Code").Value,
                        CustomItemAllowed = Convert.ToBoolean(dictionaryType.Attribute("CustomItemAllowed").Value),
                        ColorEditable = Convert.ToBoolean(dictionaryType.Attribute("ColorEditable").Value),
                        Name = dictionaryType.Attribute("Name").Value,
                        Name1 = dictionaryType.Attribute("Name1").Value,
                        Name2 = dictionaryType.Attribute("Name2").Value,
                        Name3 = dictionaryType.Attribute("Name3").Value,
                        Name4 = dictionaryType.Attribute("Name4").Value
                    };

                    foreach (XElement dictionaryItem in dictionaryType.Elements())
                    {
                        DictionaryItem newDictionaryItem = new DictionaryItem();
                        var Id = dictionaryItem.Attribute("Id").Value;
                        var enumName = dictionaryItem.Attribute("EnumName").Value;
                        newDictionaryItem.DictionaryItemValues = new List<DictionaryItemValue>();
                        foreach (XElement dictionaryItemValue in dictionaryItem.Elements())
                        {
                            var Protected = Convert.ToBoolean(dictionaryItemValue.Attribute("Protected").Value);
                            var Visible = dictionaryItemValue.Attribute("Visible") != null && !string.IsNullOrEmpty(dictionaryItemValue.Attribute("Visible").Value) ? Convert.ToBoolean(dictionaryItemValue.Attribute("Visible").Value) : true;
                            var Order = dictionaryItemValue.Attribute("Order") != null && !string.IsNullOrEmpty(dictionaryItemValue.Attribute("Order").Value) ? Convert.ToInt32(dictionaryItemValue.Attribute("Order").Value) : (int?)null;
                            var Code = dictionaryItemValue.Attribute("Code") != null && !string.IsNullOrEmpty(dictionaryItemValue.Attribute("Code").Value) ? dictionaryItemValue.Attribute("Code").Value : null;
                            var Value = dictionaryItemValue.Attribute("Value") != null && !string.IsNullOrEmpty(dictionaryItemValue.Attribute("Value").Value) ? Convert.ToInt32(dictionaryItemValue.Attribute("Value").Value) : (int?)null;
                            var Color = dictionaryItemValue.Attribute("Color") != null && !string.IsNullOrEmpty(dictionaryItemValue.Attribute("Color").Value) ? dictionaryItemValue.Attribute("Color").Value : null;
                            var Environment = dictionaryItemValue.Attribute("Environment") != null && !string.IsNullOrEmpty(dictionaryItemValue.Attribute("Environment").Value) ? dictionaryItemValue.Attribute("Environment").Value : null;
                            var Name = dictionaryItemValue.Attribute("Name") != null && !string.IsNullOrEmpty(dictionaryItemValue.Attribute("Name").Value) ? dictionaryItemValue.Attribute("Name").Value : null;
                            var Name1 = dictionaryItemValue.Attribute("Name1") != null && !string.IsNullOrEmpty(dictionaryItemValue.Attribute("Name1").Value) ? dictionaryItemValue.Attribute("Name1").Value : null;
                            var Name2 = dictionaryItemValue.Attribute("Name2") != null && !string.IsNullOrEmpty(dictionaryItemValue.Attribute("Name2").Value) ? dictionaryItemValue.Attribute("Name2").Value : null;
                            var Name3 = dictionaryItemValue.Attribute("Name3") != null && !string.IsNullOrEmpty(dictionaryItemValue.Attribute("Name3").Value) ? dictionaryItemValue.Attribute("Name3").Value : null;
                            var Name4 = dictionaryItemValue.Attribute("Name4") != null && !string.IsNullOrEmpty(dictionaryItemValue.Attribute("Name4").Value) ? dictionaryItemValue.Attribute("Name4").Value : null;
                            if (string.IsNullOrEmpty(Name))
                                throw new ArgumentNullException(string.Format("{0}: hiányzik a Name", dictionaryGroup.Code));
                            newDictionaryItem.DictionaryItemValues.Add(new DictionaryItemValue
                            {
                                Name = Name,
                                Name1 = Name1,
                                Name2 = Name2,
                                Name3 = Name3,
                                Name4 = Name4,
                                EnvironmentName = Environment,
                                Protected = Protected,
                                Visible = Visible,
                                Order = Order,
                                Code = Code,
                                Value = Value,
                                Color = Color
                            });
                        }

                        if (string.IsNullOrEmpty(enumName))
                            throw new ArgumentNullException(string.Format("{0}: hiányzik a Name", dictionaryGroup.Code));
                        if (string.IsNullOrEmpty(Id))
                            throw new ArgumentNullException(string.Format("{0}_{1}: hiányzik az Id", dictionaryGroup.Code, enumName));

                        newDictionaryItem.Id = Convert.ToInt32(Id);
                        newDictionaryItem.EnumName = enumName;

                        dictionaryGroup.Items.Add(newDictionaryItem);
                    }

                    DictionaryItemGroups.Add(dictionaryGroup);
                }
            }
            else
            {
                throw new FileNotFoundException(fileName);
            }
        }
    }
}
