﻿using System;
using System.Collections;
using System.IO;
using System.Linq;
using System.Text;
using JetBrains.Annotations;

namespace Sda.DictionaryItemEnumGenerator
{
    public class InputData
    {
        [CanBeNull]
        public string DictionaryItemGroupName { get; set; }

        [CanBeNull]
        public string TableName { get; set; }

        public bool Protected { get; set; }

        /// <summary>
        /// Csomag neve az UML-ben.
        /// </summary>
        [CanBeNull]
        public string Package { get; set; }
    }

    public class InputDataCollection : IEnumerable
    {
        readonly ArrayList _set = new ArrayList();

        public int Count
        {
            get
            {
                return _set.Count;
            }
        }

        public void Add([NotNull] InputData value)
        {
            if (value == null)
                throw new ArgumentNullException("value");

            _set.Add(value);
        }

        public void Clear()
        {
            _set.Clear();
        }

        public InputData this[int index]
        {
            get
            {
                return (InputData)_set[index];
            }
        }

        [NotNull]
        public IEnumerator GetEnumerator()
        {
            return _set.GetEnumerator();
        }

        public void Load([NotNull] string filename)
        {
            if (filename == null)
                throw new ArgumentNullException("filename");

            if (File.Exists(filename))
            {
                Clear();
                FileStream stream = File.OpenRead(filename);
                byte[] buffer = new byte[stream.Length];
                stream.Read(buffer, 0, buffer.Length);
                string filecontent = Encoding.UTF8.GetString(buffer);
                string[] rows = filecontent.Split(new[] { '\n' });
                foreach (InputData data in
                    from row in rows
                    let row2 = row.TrimEnd(new[] { '\r', '\n' })
                    where row.IndexOf(";", StringComparison.Ordinal) > -1
                    select row2.Split(new[] { ';' }) into values
                    select new InputData
                    {
                        DictionaryItemGroupName = values[0],
                        TableName = values[1],
                        Protected = Convert.ToBoolean(values[2]),
                        Package = values[3]
                    })
                {
                    Add(data);
                }
            }
            else
            {
                throw new FileNotFoundException(filename);
            }
        }
    }
}





