﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using Microsoft.VisualStudio.TextTemplating;
using Sda.Modeling.TextTemplating;

namespace Sda.Modeling.Runtime
{
    public class Generator
    {
        public event EventHandler<LogEventArgs> LogMessageHandler;

        public void Run(string output, string[] inputs)
        {
            StringBuilder templateContent;
            using (var stream = Assembly.GetExecutingAssembly().GetManifestResourceStream("Sda.Modeling.Runtime.Model.t4"))
            {
                Debug.Assert(stream != null, "stream != null");
                using (var reader = new StreamReader(stream))
                {
                    templateContent = new StringBuilder(reader.ReadToEnd());
                }
            }
            var files = new List<string>();
            Debug.Assert(inputs != null, "inputs != null");
            foreach (var input in inputs.Where(input => !string.IsNullOrEmpty(input)))
            {
                if (!File.Exists(input))
                {
                    throw new ArgumentException("One of the files was not found (" + input + ")!", "inputs");
                }
                files.Add(Path.GetFullPath(input));
            }
            var engine = new Engine();
            var host =
#if DEBUG
                new TextTemplatingEngineHost { TemplateFile = @"..\..\Model.t4", SourceFiles = files };
#else
                new TextTemplatingEngineHost { TemplateFile = ".", SourceFiles = files };
#endif
            if (LogMessageHandler != null)
            {
                host.LogMessageHandler += LogMessageHandler;
            }
            Debug.Assert(output != null, "output != null");
            try
            {
                File.WriteAllText(
                    output,
                    engine.ProcessTemplate(
                        templateContent.ToString(),
                        host),
                    Encoding.UTF8);
            }
            catch (Exception exp)
            {
                if (LogMessageHandler != null)
                {
                    LogMessageHandler(
                        this,
                        new LogEventArgs
                        {
                            Type = TraceEventType.Error,
                            Message = exp.ToString()
                        });
                }
                throw;
            }
        }
    }
}
