﻿using System;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Linq;
using Microsoft.VisualStudio.TextTemplating;

namespace Sda.Modeling.TextTemplating
{
    public sealed class TextTemplatingEngineHost : ITextTemplatingEngineHost
    {
        public event EventHandler<LogEventArgs> LogMessageHandler;

        public object GetHostOption(string optionName)
        {
            switch (optionName)
            {
                case "CacheAssemblies":
                    return true;
                default:
                    return null;
            }
        }

        public bool LoadIncludeText(string requestFileName, out string content, out string location)
        {
            content = "";
            location = "";
            if (!File.Exists(requestFileName))
            {
                return false;
            }
            content = File.ReadAllText(requestFileName);
            return true;
        }

        public void LogErrors(CompilerErrorCollection errors)
        {
            if (LogMessageHandler == null)
            {
                return;
            }
            foreach (CompilerError error in errors)
            {
                LogMessageHandler(this, new LogEventArgs { Type = TraceEventType.Error, Message = error.ErrorText });
            }
        }

        public AppDomain ProvideTemplatingAppDomain(string content)
        {
            return AppDomain.CurrentDomain;
        }

        public string ResolveAssemblyReference(string assemblyReference)
        {
            if (File.Exists(assemblyReference))
            {
                return assemblyReference;
            }
            string candidate = Path.Combine(Path.GetDirectoryName(TemplateFile ?? ".") ?? ".", assemblyReference);
            return File.Exists(candidate) ? candidate : "";
        }

        public Type ResolveDirectiveProcessor(string processorName)
        {
            if (string.Equals(processorName, "SdaModelDirectiveProcessor", StringComparison.OrdinalIgnoreCase))
            {
                return typeof(SdaModelDirectiveProcessor);
            }
            throw new ArgumentOutOfRangeException(processorName);
        }

        public string ResolveParameterValue(string directiveId, string processorName, string parameterName)
        {
            return String.Empty;
        }

        public string ResolvePath(string path)
        {
            if (path == null)
            {
                throw new ArgumentNullException("path");
            }
            if (File.Exists(path))
            {
                return path;
            }
            string candidate = Path.Combine(Path.GetDirectoryName(TemplateFile ?? ".") ?? ".", path);
            return File.Exists(candidate) ? candidate : path;
        }

        public void SetFileExtension(string extension)
        {
        }

        public void SetOutputEncoding(Encoding encoding, bool fromOutputDirective)
        {
        }

        public IList<string> StandardAssemblyReferences
        {
            get
            {
                return new[]
                {
                    typeof( String ).Assembly.Location,
                    typeof( CodeGenerator ).Assembly.Location,
                    typeof( Enumerable ).Assembly.Location,
                    typeof( XmlDocument ).Assembly.Location,
                    typeof( XDocument ).Assembly.Location
                };
            }
        }

        public IList<string> StandardImports
        {
            get
            {
                return new[]
                {
                    "System",
                    "System.Collections.Generic",
                    "System.Linq",
                    "System.Xml",
                    "System.Xml.Linq"
                };
            }
        }

        public IList<string> SourceFiles { get; set; }

        public string TemplateFile { get; set; }
    }
}
